
const { HfInference }  = require('@huggingface/inference');
const express = require('express');
const path = require('path');
const fetch = require('node-fetch');
const cors = require('cors');
const sharp = require('sharp');
const fs = require('fs').promises;
const app = express();

const hf = new HfInference(process.env.HUGGINGFACE_API_KEY);
const MODEL = 'stabilityai/stable-diffusion-xl-base-1.0';

// Enable CORS for all routes
app.use(cors());

// Serve static files from the 'public' directory
app.use(express.static(path.join(__dirname, 'public')));

// Parse JSON bodies
app.use(express.json());

let rateLimitExpiry = 0;
let isRateLimited = false;

// Create a directory for archived images if it doesn't exist
const archivedImagesDir = path.join(__dirname, 'archived_images');
fs.mkdir(archivedImagesDir, { recursive: true });

// Route to generate images using Stable Diffusion
app.post('/api/generate-image', async (req, res) => {
    const { prompt } = req.body;
    if (!prompt) {
        return res.status(400).send('Prompt is required');
    }

    try {
        const blob = await hf.textToImage({ inputs: prompt, model: MODEL, parameters: { negative_prompt: 'blurry' } });
        const buffer = Buffer.from(await blob.arrayBuffer())
        const base64 = buffer.toString('base64');

        res.json({ image: `data:image/jpeg;base64,${base64}` });
    } catch (error) {
        console.error('Error generating image:', error);
        res.status(500).send(`Error generating image: ${error.message}`);
    }
});

// Route to get archived images
app.get('/api/archived-images', async (req, res) => {
    try {
        const files = await fs.readdir(archivedImagesDir);
        const imageUrls = files.map(file => `/archived_images/${file}`);
        res.json(imageUrls);
    } catch (error) {
        console.error('Error reading archived images:', error);
        res.status(500).send('Error retrieving archived images');
    }
});

// Route to add an image to the archive
app.post('/api/archive-image', async (req, res) => {
    const { imageUrl } = req.body;
    if (!imageUrl) {
        return res.status(400).send('Image URL is required');
    }

    try {
        const response = await fetch(imageUrl);
        if (response.ok) {
            const buffer = await response.buffer();
            const fileName = `image_${Date.now()}.png`;
            await fs.writeFile(path.join(archivedImagesDir, fileName), buffer);
            res.json({ success: true, fileName });
        } else {
            res.status(response.status).send('Failed to fetch image');
        }
    } catch (error) {
        console.error('Error archiving image:', error);
        res.status(500).send('Error archiving image');
    }
});

// Route to delete an archived image
app.delete('/api/archived-images/:fileName', async (req, res) => {
    const { fileName } = req.params;
    try {
        await fs.unlink(path.join(archivedImagesDir, fileName));
        res.json({ success: true });
    } catch (error) {
        console.error('Error deleting archived image:', error);
        res.status(500).send('Error deleting archived image');
    }
});

// Serve archived images statically
app.use('/archived_images', express.static(archivedImagesDir));

const PORT = process.env.PORT || 3000;
app.listen(PORT, () => {
    console.log(`Server is running on port ${PORT}`);
});
